/*
 YamCha -- Yet Another Multipurpose CHunk Annotator

 $Id: param.h,v 1.16 2002/10/15 16:33:36 taku-ku Exp $;

 Copyright (C) 2001-2002  Taku Kudo <taku-ku@is.aist-nara.ac.jp>
 All rights reserved.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later verjsion.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the
 Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.
*/

#ifndef _YAMCHA_BASE_PARAM_H
#define _YAMCHA_BASE_PARAM_H

#include <string.h>
#include <map>
#include <string>
#include <vector>
#include <strstream>
#include <stdexcept>
#include <iostream>
#include <cstdlib>

namespace YamCha {

  struct Option {
    const char *name;
    char short_name;
    int has_arg;
  };

  class Param 
  {
  private:
    std::map <std::string, std::string> conf;
    std::vector <std::string> rest;
    std::string systemName;
    std::string _what;
    Option *opts;
    std::ostream *os; 

  public:
    bool open   (int,  char**);
    bool open   (const char*);
    bool load   (const char*);
    bool getopt (int,  char**);

    void clear ()                                   { conf.clear (); rest.clear (); };
    const std::vector <std::string>& getRestArg ()  { return rest; };
    const char* getSystemName ()                    { return systemName.c_str(); };
    const char* what ()                             { return _what.c_str(); };
    void  setHelpStream (std::ostream &_os)         { os = &_os; }

    const std::string getProfileString (const char* key, bool check = false)//    
    {
      std::string val = conf[std::string(key)];
       if (check && val == "")
	 throw std::runtime_error (std::string("Param::getProfileString(): [") + key + "] is not defined.");
       return val;
    }
    
    int getProfileInt (const char* key, bool check = false)
    {
       std::string val = conf[std::string(key)];
       if (check && val == "") 
	 throw std::runtime_error (std::string("Param::getProfileString(): [") + key + "] is not defined.");
       return std::atoi (val.c_str());
    }

    void setProfile (const char* key, const char* value, bool rewrite = false)
    {
       std::string keys2 = std::string(key);
      if (rewrite || (! rewrite && conf[keys2] == "")) conf[keys2] = value;
    }

    void setProfile (const char* key, int i, bool rewrite = true)
    {
      std::string keys2 = std::string(key);
      if (rewrite || (! rewrite && conf[keys2] == "")) {
	std::strstream ss; ss << i; 
	conf[std::string(key)] = ss.str();
      }
    }
   
    Param (): os(0) {};
    ~Param () {};
  };
}

#endif

